"""
Author: Mayank Arora (hotshot07)
"""
from ._base_class import BaseClass
from ._timer import Timer
import copy
import matplotlib.pyplot as plt
import matplotlib.animation as animation


class BubbleSort(BaseClass):
    """
    In Bubble Sort  we look at pairs of adjacent elements in an array,
    one pair at a time, and swap their positions if the first element is
    larger than the second, or simply move on if it isn't.
    """

    def __init__(self, datalist):
        super().__init__(datalist)
        self.__datalist = datalist

    # A generator for the ascending bubble sort algorithm
    def __ascending_sort_algo(self):
        _asc_list = copy.deepcopy(self.__datalist)
        _length_of_list = len(_asc_list)
        _has_swapped = True
        _number_of_iterations = 0

        while _has_swapped:
            _has_swapped = False

            for i in range(_length_of_list - _number_of_iterations - 1):
                if _asc_list[i] > _asc_list[i + 1]:
                    _asc_list[i], _asc_list[i + 1] = _asc_list[i + 1], _asc_list[i]
                    _has_swapped = True

            _number_of_iterations += 1
            yield _asc_list

    # A generator for the descending bubble sort algorithm
    def __descending_sort_algo(self):
        _desc_list = copy.deepcopy(self.__datalist)
        _length_of_list = len(_desc_list)
        _has_swapped = True
        _number_of_iterations = 0

        while _has_swapped:
            _has_swapped = False

            for i in range(_length_of_list - _number_of_iterations - 1):
                if _desc_list[i] < _desc_list[i + 1]:
                    _desc_list[i], _desc_list[i + 1] = _desc_list[i + 1], _desc_list[i]

                    _has_swapped = True

            _number_of_iterations += 1
            yield _desc_list

    # The function that is called by sort method
    def __sort_it(self, reverse, steps):
        _iteration_dict = {}
        iterations = 0

        if not reverse:
            for _yielded_list in self.__ascending_sort_algo():
                iterations += 1
                _iteration_dict[iterations] = copy.deepcopy(_yielded_list)
        else:
            for _yielded_list in self.__descending_sort_algo():
                iterations += 1
                _iteration_dict[iterations] = copy.deepcopy(_yielded_list)

        if steps:
            print()
            print("Iteration    List")
            for _iter, _list in _iteration_dict.items():
                print("    " + str(_iter) + "        " + str(_list))

            print()
        return _iteration_dict

    # Evaluating time of ascending bubble sort
    # Didn't use generators as I dont wanna waste time in dealing
    # with overheads
    def __time_eval_asc(self, iterations):
        _time_list = copy.deepcopy(self.__datalist)
        _length_of_list = len(_time_list)
        _timing_list = []

        while iterations:
            _has_swapped = True
            _number_of_iterations = 0

            timer = Timer()
            timer.start()

            while _has_swapped:
                _has_swapped = False

                for i in range(_length_of_list - _number_of_iterations - 1):
                    if _time_list[i] > _time_list[i + 1]:
                        _time_list[i], _time_list[i + 1] = _time_list[i + 1], _time_list[i]

                        _has_swapped = True

                _number_of_iterations += 1

            stop = timer.stop()
            _timing_list.append(stop)
            iterations -= 1
            _time_list = copy.deepcopy(self.__datalist)

        return _timing_list

    # Evaluating time of descending bubble sort
    def __time_eval_desc(self, iterations):
        _time_list = copy.deepcopy(self.__datalist)
        _length_of_list = len(_time_list)
        _timing_list = []

        while iterations:
            _has_swapped = True
            _number_of_iterations = 0

            timer = Timer()
            timer.start()

            while _has_swapped:
                _has_swapped = False

                for i in range(_length_of_list - _number_of_iterations - 1):
                    if _time_list[i] < _time_list[i + 1]:
                        _time_list[i], _time_list[i + 1] = _time_list[i + 1], _time_list[i]

                        _has_swapped = True

                _number_of_iterations += 1

            stop = timer.stop()
            _timing_list.append(stop)
            iterations -= 1
            _time_list = copy.deepcopy(self.__datalist)

        return _timing_list

    # Helper function for the animation
    def __update_fig(self, _vis_list, rects, iteration, text):
        for rect, val in zip(rects, _vis_list):
            rect.set_height(val)
        iteration[0] += 1
        text.set_text("# of operations: {}".format(iteration[0]))

    def sort(self, reverse=False, steps=False):
        _sorted_object = self.__sort_it(reverse, steps)
        return list(_sorted_object.values())[-1]

    def evaluate(self, reverse=False, iterations=1):
        if reverse:
            _timing_list = self.__time_eval_desc(iterations)
        else:
            _timing_list = self.__time_eval_asc(iterations)

        _minimum_time = str("{:.10f}s".format(min(_timing_list)))
        _maximum_time = str("{:.10f}s".format(max(_timing_list)))
        _average_time = str("{:.10f}s".format(sum(_timing_list) / iterations))

        eval_dict = {
            "Minimum Time:": _minimum_time,
            "Maximum Time:": _maximum_time,
            "Average Time:": _average_time
        }
        return eval_dict

    def visualize(self, reverse=False):
        _vis_list = copy.deepcopy(self.__datalist)
        plt.style.use('dark_background')
        fig, ax = plt.subplots()
        ax.set_title("Bubble Sort")
        bar_rects = ax.bar(range(len(_vis_list)), _vis_list, align="edge")
        text = ax.text(0.02, 0.95, "", transform=ax.transAxes)
        iteration = [0]
        if not reverse:
            anim = animation.FuncAnimation(fig, func=self.__update_fig,
                                           fargs=(bar_rects, iteration, text), frames=self.__ascending_sort_algo(), interval=250,
                                           repeat=False)
        else:
            anim = animation.FuncAnimation(fig, func=self.__update_fig,
                                           fargs=(bar_rects, iteration, text), frames=self.__descending_sort_algo(), interval=250,
                                           repeat=False)

        plt.show()



# This module contains the functions
# common to sorting algorithms

class BaseClass:
    def __init__(self, datalist):
        if not isinstance(datalist, list):
            raise TypeError('parameter must be a list datatype')

        if not datalist:
            raise ValueError('parameter has invalid value')

        for _element in datalist:
            if not isinstance(_element, int):
                raise TypeError('parameter can only contain int datatype')



import matplotlib.pyplot as plt
import matplotlib.animation as animation
import numpy as np
import random


class AnimateAlgorithm:
    def __init__(self):
        fig, ax = plt.subplots()

        ax.set_title("Bubble Sort")

        bar_rects = ax.bar(range(len()), my_list, align="edge")

        text = ax.text(0.02, 0.95, "", transform=ax.transAxes)

        iteration = [0]

    def update_fig(my_list, rects, iteration):
        for rect, val in zip(rects, my_list):
            rect.set_height(val)
            iteration[0] += 1
            text.set_text("# of operations: {}".format(iteration[0]))

    anim = animation.FuncAnimation(fig, func=update_fig,
                                   fargs=(bar_rects, iteration), frames=bubblesort(my_list), interval=250,
                                   repeat=False)

    plt.show()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['algovis', 'algovis.sorting']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.2.1,<4.0.0']

setup_kwargs = {
    'name': 'algovis',
    'version': '0.1.3',
    'description': 'Python library for visualising algorithms',
    'long_description': '![Alogvis]( /algovis_img.PNG?raw=true)\n\nAlgovis is a library with which you can learn how to code data structures and algorithms in Python\n\nIt will have most of the prominent sorting, searching and graph algorithms with visualisations.\n\nCurrently the library has\n\n# Sorting\n\n- Bubble Sort\n\n# Getting Started\n\nI would highly suggest making a virtual environment as this library is still in beta\n\nFor making a virtual environment, we can use virtualenv\n\n```python\n# installing virtualenv\n\n$pip3 install virtualenv\n\n# making a test folder\n\n$mkdir test\n\n# make it the current directory\n\n$ cd test\n\n# making a virtual env (you can replace envname with whatever name you like)\n\n$virtualenv - p python3 envname\n\n# activating it\n\n$ source envname / bin / activate\n\n# installing algovis\n\n$ pip3 install algovis\n```\n\nYou can only access algovis inside this virtual environment. To leave this virtualenv, type\n\n```python\n$deactivate\n```\n\n# Using this library\n\nI have designed the library this way so that supporting making changes to it is easy\n\n```python\n# Using bubblesort in algovis\n\n# import the sorting module from library\n\nfrom algovis import sorting\n\n# importing random library to fill it the list with random integers\n\nimport random\n\n# Making a list of 50 random integers in range of 0 to 100\n\nmy_list = [random.randint(0, 100) for i in range(50)]\n\n# Making a bubble sort class object\n# It only accepts lists, raises an exception otherwise\n\nbs_object = sorting.BubbleSort(my_list)\n\n# bs_object is now an object of the bubblesort class\n# The class has 3 functions\n\n# .sort() with 2 optional parameters reverse and steps, both boolean\n# reverse sorts the list in descending order and steps shows every iteration\n# of the bubble sort algotithm\n\n# default of reverse if False\n# default of steps is False\n# return type: Dictionary\n\ndesc_sort = bs_object.sort(reverse=True, steps=True)\n\n# .evaluate() with 2 optional parameters reverse and iterations\n# .evaluate() returns a dictionary giving a dictionary of minimum,\n# maximum and average time to sort the list in seconds\n\n# reverse option sorts it in descending order\n# default is False\n\n# iterations is the number of times you want to run the algo\n# default is 1\n\neval_algo0 = bs_object.evaluate(iterations=100)\n\neval_algo1 = bs_object.evaluate(reverse=True, iterations=500)\n\n# .visualize() makes the visualization of the list you gave getting sorted\n# in ascending order\n# it has one option, reverse which is a boolean\n\nvis_algo = bs_object.visualize(reverse=True)\n\n```\n\n    [Last updated on 16th June 2020]\n',
    'author': 'hotshot07',
    'author_email': 'aroram@tcd.ie',
    'maintainer': 'hotshot07',
    'maintainer_email': 'aroram@tcd.ie',
    'url': 'https://github.com/hotshot07/algovis',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.0,<4.0.0',
}


setup(**setup_kwargs)
